/// <reference types="node" />
import { ForgeClient, IAuthOptions } from 'forge-server-utils/dist/common';
export interface IView {
    urn: string;
    id: string;
    role: string;
    mime: string;
    resolvedUrn: string;
}
interface IPropertyDbAsset {
    tag: string;
    type: string;
    resolvedUrn: string;
}
export declare class ManifestHelper {
    protected manifest: any;
    constructor(manifest: any);
    get sharedRoot(): any;
    listViews(): IView[];
    listSharedDatabaseAssets(): IPropertyDbAsset[];
}
export declare class ViewHelper {
    protected view: any;
    protected resolvedViewUrn: string;
    constructor(view: any, resolvedViewUrn: string);
    listPrivateModelAssets(): ({
        [key: string]: {
            uri: string;
            resolvedUrn: string;
        };
    } | undefined);
    listSharedModelAssets(): ({
        [key: string]: {
            uri: string;
            resolvedUrn: string;
        };
    } | undefined);
    listPrivateDatabaseAssets(): ({
        [key: string]: {
            uri: string;
            resolvedUrn: string;
        };
    } | undefined);
    listSharedDatabaseAssets(): ({
        [key: string]: {
            uri: string;
            resolvedUrn: string;
        };
    } | undefined);
    resolveAssetUrn(assetUrn: string): string;
    getGeometryUrn(hash: string): string;
    getMaterialUrn(hash: string): string;
    getTextureUrn(hash: string): string;
}
/**
 * Client for the OTG service (https://otg.autodesk.com).
 */
export declare class Client extends ForgeClient {
    /**
     * Initializes the client with specific authentication and host address.
     * @param {IAuthOptions} auth Authentication object,
     * containing either `client_id` and `client_secret` properties (for 2-legged authentication),
     * or a single `token` property (for 2-legged or 3-legged authentication with pre-generated access token).
     * @param {string} [host="https://otg.autodesk.com"] OTG service host.
     */
    constructor(auth: IAuthOptions, host?: string);
    /**
     * Triggers processing of OTG derivatives for a specific model.
     * Note: the model must already be processed into SVF using the Model Derivative service.
     * @async
     * @param {string} urn Model Derivative model URN.
     * @param {string} [account] Optional account ID.
     * @param {boolean} [force] Optional flag to force the translation.
     * @throws Error when the request fails, for example, due to insufficient rights, or incorrect scopes.
     */
    createDerivatives(urn: string, account?: string, force?: boolean): Promise<any>;
    /**
     * Removes OTG derivatives for a specific model.
     * @async
     * @param {string} urn Model Derivative model URN.
     * @throws Error when the request fails, for example, due to insufficient rights, or incorrect scopes.
     */
    deleteDerivatives(urn: string): Promise<any>;
    /**
     * Retrieves the Model Derivative manifest augmented with OTG information.
     * @async
     * @param {string} urn Model Derivative model URN.
     * @returns {Promise<any>} Model Derivative manifest augmented with OTG information.
     * @throws Error when the request fails, for example, due to insufficient rights, or incorrect scopes.
     */
    getManifest(urn: string): Promise<any>;
    /**
     * Retrieves raw data of specific OTG asset.
     * @async
     * @param {string} urn Model Derivative model URN.
     * @param {string} assetUrn OTG asset URN, typically composed from OTG "version root" or "shared root",
     * path to OTG view JSON, etc.
     * @returns {Promise<Buffer>} Asset data.
     * @throws Error when the request fails, for example, due to insufficient rights, or incorrect scopes.
     */
    getAsset(urn: string, assetUrn: string): Promise<Buffer>;
}
export declare class SharedClient extends ForgeClient {
    protected sharding: number;
    constructor(auth: IAuthOptions, host?: string);
    getAsset(urn: string, assetUrn: string): Promise<Buffer>;
}
export {};
//# sourceMappingURL=client.d.ts.map