/// <reference types="node" />
import { IAuthOptions } from 'forge-server-utils/dist/common';
import { Client as OtgClient, SharedClient as OtgSharedClient, ViewHelper as OtgViewHelper } from './client';
import * as OTG from './schema';
import * as IMF from '../common/intermediate-format';
interface IOtgView {
    id: string;
    fragments: OTG.IFragment[];
    geometryHashes: string[];
    geometryMap: Map<string, OTG.IGeometry>;
    materialHashes: string[];
    materialMap: Map<string, OTG.IMaterial>;
}
export declare class Scene implements IMF.IScene {
    protected otg: IOtgView;
    constructor(otg: IOtgView);
    getMetadata(): IMF.IMetadata;
    getNodeCount(): number;
    getNode(id: number): IMF.Node;
    getGeometryCount(): number;
    private _parseVertexAttributes;
    private _parseTexCoords;
    private _parseNormals;
    private _parseIndices;
    private _decodeTriangleIndices;
    private _decodeLineIndices;
    getGeometry(id: number): IMF.Geometry;
    getMaterialCount(): number;
    getMaterial(id: number): IMF.Material;
    getImage(uri: string): Buffer | undefined;
}
/**
 * Additional reader options.
 */
export interface IReaderOptions {
    log?: (msg: string) => void;
}
/**
 * Experimental reader of the OTG file format (successor to SVF, with focus on geometry deduplication).
 * Missing features:
 *   - reading metadata
 *   - reading material textures
 *   - parsing line/point geometry
 */
export declare class Reader {
    protected urn: string;
    protected manifest: any;
    protected client: OtgClient;
    protected sharedClient: OtgSharedClient;
    protected log: (msg: string) => void;
    static FromDerivativeService(urn: string, guid: string, auth: IAuthOptions, options?: IReaderOptions): Promise<Reader>;
    protected constructor(urn: string, manifest: any, client: OtgClient, sharedClient: OtgSharedClient, options?: IReaderOptions);
    read(): Promise<Scene>;
    protected readView(id: string, resolvedUrn: string): Promise<IOtgView>;
    protected parseFragments(fragListUrn: string, output: OTG.IFragment[]): Promise<void>;
    protected parseGeometries(geomHashListUrn: string, otgViewHelper: OtgViewHelper, hashes: string[], map: Map<string, OTG.IGeometry>): Promise<void>;
    protected parseGeometry(otgViewHelper: OtgViewHelper, hash: string, map: Map<string, OTG.IGeometry>): Promise<void>;
    protected parseMaterials(matHashListUrn: string, otgViewHelper: OtgViewHelper, hashes: string[], map: Map<string, OTG.IMaterial>): Promise<void>;
    protected parseMaterial(otgViewHelper: OtgViewHelper, hash: string, map: Map<string, OTG.IMaterial>): Promise<void>;
}
export {};
//# sourceMappingURL=reader.d.ts.map